﻿#include <GUIConstantsEx.au3>
#include <GuiListView.au3>
#include <WindowsConstants.au3>

Global $sFile = @ScriptDir & '\autocomplete.txt'
Global $sData = FileRead($sFile)
Global $aWords = StringSplit($sData, @CRLF, 3)
Global $iTimer, $iDiff

; Zur Auswahl der Sortiermethode (Achtung! InsertionSort dauert sehr lange!)
; $bQuicksort = True gleich Quicksort (bei mir: 1.865 s)
; $bQuicksort = False gleich InsertionSort (bei mir: 287.283 s also fast 5 Minuten)
Global $bQuicksort = True
Switch $bQuicksort
	Case True
		$iTimer = TimerInit()
		_QSSortLength($aWords)
		$iDiff = TimerDiff($iTimer) / 1000
	Case False
		$iTimer = TimerInit()
		_QSInsertionSort($aWords)
		$iDiff = TimerDiff($iTimer) / 1000
EndSwitch
;~ _ArrayReverse($aWords)
ConsoleWrite(StringFormat('Wörter: %i\r\nZeit: %.03f s\r\n', UBound($aWords), $iDiff))
_ArrayShow($aWords)

; Funktion fuer den Stringvergleich
Func _QSStringCmp(Const $sD1, Const $sD2)
	Local $1 = StringLen($sD1), $2 = StringLen($sD2)
	If $1 = $2 Then Return StringCompare($sD1, $sD2, 1) >= 0
	Return $1 > $2
EndFunc   ;==>_QSStringCmp

; Rekursive Quicksort-Funktion
Func _QSSortLength(ByRef $aData, $left = 0, $right = UBound($aWords) - 1)
	If $right - $left >= 20 Then ; bei 20 und mehr Elementen, Quicksort benutzen
		If $left < $right Then
			Local $pivot = _QSSplitSwap($aData, $left, $right)
			_QSSortLength($aData, $left, $pivot - 1)
			_QSSortLength($aData, $pivot + 1, $right)
		EndIf
	Else ; bei weniger als 20 (noch verbleibenden) Elementen, ist InsertionSort schneller
		_QSInsertionSort($aData, $left, $right)
	EndIf
EndFunc   ;==>_QSSortLength

Func _QSSplitSwap(ByRef $aData, $left, $right)
	Local $i = $left, $j = $right - 1, $pivot = $aData[$right], $h
	Do
		While _QSStringCmp($aData[$i], $pivot) And $i < $right
			$i += 1
		WEnd
		While _QSStringCmp($pivot, $aData[$j]) And $j > $left
			$j -= 1
		WEnd
		If $i < $j Then
			$h = $aData[$i]
			$aData[$i] = $aData[$j]
			$aData[$j] = $h
		EndIf
	Until $i >= $j
	$aData[$right] = $aData[$i]
	$aData[$i] = $pivot
	Return $i
EndFunc   ;==>_QSSplitSwap

; InsertionSort-Funktion
Func _QSInsertionSort(ByRef $aData, $iStart = 0, $iEnd = UBound($aData) - 1)
	Local $sVal, $j
	For $i = $iStart To $iEnd
		$sVal = $aData[$i]
		$j = $i
		While ($j > $iStart) And _QSStringCmp($sVal, $aData[$j - 1])
			$aData[$j] = $aData[$j - 1]
			$j -= 1
		WEnd
		$aData[$j] = $sVal
	Next
EndFunc   ;==>_QSInsertionSort

; Funktion zum Anzeigen des Arrays
Func _ArrayShow(ByRef $aData, $sTitle = 'ArrayShow')
	Local $hGui = GUICreate($sTitle, 540, @DesktopHeight - 140, -1, 10, BitOR($GUI_SS_DEFAULT_GUI, $WS_SIZEBOX))
	Local $idLV = GUICtrlCreateListView('Row|Data', 5, 5, 530, @DesktopHeight - 150)
	GUICtrlSetFont(-1, 12, 400, 0, 'Courier New')
	GUICtrlSetResizing(-1, $GUI_DOCKBORDERS)
	_GUICtrlListView_JustifyColumn($idLV, 0, 1)
	_GUICtrlListView_BeginUpdate($idLV)
	For $i = 0 To UBound($aData) - 1
		GUICtrlCreateListViewItem($i & '|"' & $aData[$i] & '"', $idLV)
	Next
	_GUICtrlListView_SetColumnWidth($idLV, 0, $LVSCW_AUTOSIZE)
	_GUICtrlListView_SetColumnWidth($idLV, 1, $LVSCW_AUTOSIZE)
	_GUICtrlListView_EndUpdate($idLV)
	GUISetState()
	Do
	Until GUIGetMsg() = -3
	GUIDelete($hGui)
EndFunc   ;==>_ArrayShow
